module ethernet_top #(
    parameter N_PORTS = 4,
    parameter TARGET = "XILINX"
)(
    input   i_clk_100,
    input   i_rst_100,

    input   i_clk_125,
    input   i_clk_125_90,
    input   i_rst_125,

    axis_intf.MASTER                m_eth_axis[N_PORTS],
    axis_intf.SLAVE                 s_eth_axis[N_PORTS],

    input  wire [N_PORTS-1:0]       rgmii_rx_clk,
    input  wire [N_PORTS*4-1:0]     rgmii_rxd,
    input  wire [N_PORTS-1:0]       rgmii_rx_ctl,
    output wire [N_PORTS-1:0]       rgmii_tx_clk,
    output wire [N_PORTS*4-1:0]     rgmii_txd,
    output wire [N_PORTS-1:0]       rgmii_tx_ctl,

    axil_intf.SLAVE                 s_eth_reg_axil
);

// top level regs and axi->cpuif
ethernet_top_regs_pkg::ethernet_top_regs__in_t eth_hwif_in;
ethernet_top_regs_pkg::ethernet_top_regs__out_t eth_hwif_out;

ethernet_top_regs u_eth_regs(
    .clk                    (i_clk_100),
    .rst                    (i_rst_100),

    .s_axil_awready         (s_eth_reg_axil.awready),
    .s_axil_awvalid         (s_eth_reg_axil.awvalid),
    .s_axil_awaddr          (s_eth_reg_axil.awaddr[6:0]),
    .s_axil_awprot          ('0),
    .s_axil_wready          (s_eth_reg_axil.wready),
    .s_axil_wvalid          (s_eth_reg_axil.wvalid),
    .s_axil_wdata           (s_eth_reg_axil.wdata),
    .s_axil_wstrb           (s_eth_reg_axil.wstrb),
    .s_axil_bready          (s_eth_reg_axil.bready),
    .s_axil_bvalid          (s_eth_reg_axil.bvalid),
    .s_axil_bresp           (s_eth_reg_axil.bresp),
    .s_axil_arready         (s_eth_reg_axil.arready),
    .s_axil_arvalid         (s_eth_reg_axil.arvalid),
    .s_axil_araddr          (s_eth_reg_axil.araddr[6:0]),
    .s_axil_arprot          ('0),
    .s_axil_rready          (s_eth_reg_axil.rready),
    .s_axil_rvalid          (s_eth_reg_axil.rvalid),
    .s_axil_rdata           (s_eth_reg_axil.rdata),
    .s_axil_rresp           (s_eth_reg_axil.rresp),

    .hwif_in                (eth_hwif_in),
    .hwif_out               (eth_hwif_out)
);

cpuif_intf #(.ADDR_WIDTH(5)) mac_cpuif[N_PORTS]();


// mac generate
generate
    for (genvar i = 0; i < N_PORTS; i++) begin : MAC_GENERATE
        assign mac_cpuif[i].req            = eth_hwif_out.mac[i].req;
        assign mac_cpuif[i].req_is_wr      = eth_hwif_out.mac[i].req_is_wr;
        assign mac_cpuif[i].addr           = eth_hwif_out.mac[i].addr;
        assign mac_cpuif[i].wr_data        = eth_hwif_out.mac[i].wr_data;
        assign mac_cpuif[i].wr_biten       = eth_hwif_out.mac[i].wr_biten;
        assign eth_hwif_in.mac[i].rd_ack   = mac_cpuif[i].rd_ack;
        assign eth_hwif_in.mac[i].rd_data  = mac_cpuif[i].rd_data;
        assign eth_hwif_in.mac[i].wr_ack   = mac_cpuif[i].wr_ack;

        mac_wrapper #(
            .TARGET(TARGET),
            .USE_CLK90("FALSE")
        ) u_mac_wrapper (
            .i_clk_100      (i_clk_100),
            .i_rst_100      (i_rst_100),
            .i_clk_125      (i_clk_125),
            .i_clk_125_90   (i_clk_125_90),
            .i_rst_125      (i_rst_125),

            .m_eth_axis     (m_eth_axis[i]),
            .s_eth_axis     (s_eth_axis[i]),

            .rgmii_rx_clk   (rgmii_rx_clk[i]),
            .rgmii_rxd      (rgmii_rxd[i*4+:4]),
            .rgmii_rx_ctl   (rgmii_rx_ctl[i]),
            .rgmii_tx_clk   (rgmii_tx_clk[i]),
            .rgmii_txd      (rgmii_txd[i*4+:4]),
            .rgmii_tx_ctl   (rgmii_tx_ctl[i]),

            .s_reg_cpuif    (mac_cpuif[i])
        );
    end
endgenerate


endmodule