module axis_arb_mux_wrap #(
    parameter S_COUNT = 4,
    parameter ARB_TYPE_ROUND_ROBIN = 1
) (
    input                   i_clk,
    input                   i_rst,

    axis_intf.SLAVE         s_axis[S_COUNT],
    axis_intf.MASTER        m_axis
);

`ifdef ASSERTIONS
initial begin
    if s_axis[0].DATA_WIDTH != m_axis.DATA_WIDTH) begin
        $error("Arb input and output widths are not equal")!
    end
end
`endif

localparam DATA_WIDTH = m_axis.DATA_WIDTH;
localparam USER_WIDTH = 1;
localparam KEEP_WIDTH = ((DATA_WIDTH+7)/8);

logic [S_COUNT*DATA_WIDTH-1:0]  s_axis_tdata;
logic [S_COUNT*KEEP_WIDTH-1:0]  s_axis_tkeep;
logic [S_COUNT-1:0]             s_axis_tvalid;
logic [S_COUNT-1:0]             s_axis_tready;
logic [S_COUNT-1:0]             s_axis_tlast;
logic [S_COUNT*USER_WIDTH-1:0]  s_axis_tuser;

generate
    for (genvar i = 0; i < S_COUNT; i++) begin
        assign s_axis_tdata[i*DATA_WIDTH+:DATA_WIDTH] = s_axis[i].tdata;
        assign s_axis_tkeep[i*KEEP_WIDTH+:KEEP_WIDTH] = s_axis[i].tkeep;
        assign s_axis_tvalid[i] = s_axis[i].tvalid;
        assign s_axis[i].tready = s_axis_tready[i];
        assign s_axis_tlast[i] = s_axis[i].tlast;
        assign s_axis_tuser[i*USER_WIDTH+:USER_WIDTH] = s_axis[i].tuser;
    end
endgenerate

localparam M_ID_WIDTH = $clog2(S_COUNT);

logic [M_ID_WIDTH-1:0] m_tid;
assign m_axis.tid = {6'h0, m_tid};

axis_arb_mux #(
    .S_COUNT(S_COUNT),
    .DATA_WIDTH(DATA_WIDTH),
    .ARB_TYPE_ROUND_ROBIN(ARB_TYPE_ROUND_ROBIN),
    .ID_ENABLE(1),
    .S_ID_WIDTH(0),
    .UPDATE_TID(1)
) u_axis_arb_mux (
    .clk            (i_clk),
    .rst            (i_rst),

    .s_axis_tdata   (s_axis_tdata),
    .s_axis_tkeep   (s_axis_tkeep),
    .s_axis_tvalid  (s_axis_tvalid),
    .s_axis_tready  (s_axis_tready),
    .s_axis_tlast   (s_axis_tlast),
    .s_axis_tid     ('0),
    .s_axis_tdest   ('0),
    .s_axis_tuser   (s_axis_tuser),

    .m_axis_tdata   (m_axis.tdata),
    .m_axis_tkeep   (m_axis.tkeep),
    .m_axis_tvalid  (m_axis.tvalid),
    .m_axis_tready  (m_axis.tready),
    .m_axis_tlast   (m_axis.tlast),
    .m_axis_tid     (m_tid),
    .m_axis_tdest   (m_axis.tdest),
    .m_axis_tuser   (m_axis.tuser)
);

endmodule