module input_mux #(
    parameter ETHERNET_COUNT = 4
) (
    input                   core_clk,
    input                   core_rst,

    input                   eth_clk,
    input                   eth_rst,


    axis_intf.SLAVE         s_eth_axis[ETHERNET_COUNT],

    axis_intf.MASTER        m_core_axis
);

localparam CORE_WIDTH = m_core_axis.DATA_WIDTH;


axis_intf #(.DATA_WIDTH(CORE_WIDTH)) eth_axis_upscaled[ETHERNET_COUNT]();

// This should come from the switch_core level
cpuif_intf #(.ADDR_WIDTH(6)) fifo_reg_cpuif[ETHERNET_COUNT]();

generate
    for (genvar i = 0; i < ETHERNET_COUNT; i++) begin : gen_upscale_fifo
        upscale_saf_wrapper #(
            .DEPTH(4096)
        ) u_upscale_saf_wrapper (
            .s_clk          (eth_clk),
            .s_rst          (eth_rst),

            .m_clk          (core_clk),
            .m_rst          (core_rst),

            .s_axis         (s_eth_axis[i]),
            .m_axis         (eth_axis_upscaled[i]),

            .rclr           ('0),
            .reg_cpuif      (fifo_reg_cpuif[i])
        );
    end
endgenerate


axis_arb_mux_wrap #(
    .S_COUNT(ETHERNET_COUNT)
) u_axis_arb_mux_wrap (
    .i_clk      (core_clk),
    .i_rst      (core_rst),

    .s_axis     (eth_axis_upscaled),
    .m_axis     (m_core_axis)
);

// we should convert to big endian here

endmodule