module mac_lookup(
    input core_clk,
    input core_rst,

    axis_intf.SLAVE s_axis,
    axis_intf.MASTER m_axis
);

logic mac_valid;

logic [47:0] src_mac;
logic [47:0] dst_mac;

logic [7:0] dst_port;
logic       dst_port_valid;

axis_intf #(.DATA_WIDTH(64), .ID_WIDTH(8)) delay_axis();
axis_intf #(.DATA_WIDTH(64), .ID_WIDTH(8)) done_axis();

mac_separate u_mac_separate (
    .core_clk       (core_clk),
    .core_rst       (core_rst),

    .s_axis         (s_axis),
    .m_axis         (delay_axis),

    .o_src_mac      (src_mac),
    .o_dst_mac      (dst_mac),

    .o_mac_valid    (mac_valid)
);

// packet shift register
axis_pipeline_register_wrapper #(
    .REG_TYPE(1),
    .LENGTH(3)
) u_axis_pipe (
    .clk        (core_clk),
    .rst        (core_rst),

    .s_axis     (delay_axis),
    .m_axis     (done_axis)
);

mac_table #(
    .TABLE_ADDR_WIDTH(9)
) u_mac_table (
    .core_clk       (core_clk),
    .core_rst       (core_rst),

    .i_access       (mac_valid),

    .i_dst_mac      (dst_mac),
    .o_dst_port     (dst_port),
    .o_dst_port_valid(dst_port_valid),

    .i_src_mac      (src_mac),
    .i_src_port     (s_axis.tid)
);

logic [7:0] dst_port_reg;

always @(posedge core_clk) begin
    if (core_rst) begin
        dst_port_reg <= '1;
    end else if (dst_port_valid) begin
        dst_port_reg <= dst_port;
    end
end

assign m_axis.tdata        = done_axis.tdata;
assign m_axis.tkeep        = done_axis.tkeep;
assign m_axis.tvalid       = done_axis.tvalid;
assign done_axis.tready = m_axis.tready;
assign m_axis.tlast        = done_axis.tlast;
assign m_axis.tdest        = dst_port_reg;
assign m_axis.tid          = done_axis.tid;
assign m_axis.tuser        = done_axis.tuser;

endmodule