module mac_table #(
    parameter TABLE_ADDR_WIDTH = 9
)(
    input  core_clk,
    input  core_rst,

    input  i_access,

    // lookup destination
    input  logic [47:0] i_dst_mac,
    output logic [7:0]  o_dst_port,
    output logic        o_dst_port_valid,

    // write source
    input  logic [47:0] i_src_mac,
    input  logic [7:0]  i_src_port
);

logic [(8+48)-1:0] mac_table_mem [2**TABLE_ADDR_WIDTH];

logic [31:0] src_mac_hash, dst_mac_hash;

logic [1:0] access_d;

logic [TABLE_ADDR_WIDTH-1:0] wr_addr, wr_addr_next;
logic [TABLE_ADDR_WIDTH-1:0] rd_addr, rd_addr_next;

lfsr #(
    .LFSR_WIDTH(32),
    .LFSR_POLY(32'h4c11db7),
    .LFSR_CONFIG("GALOIS"),
    .LFSR_FEED_FORWARD(0),
    .REVERSE(1),
    .DATA_WIDTH(48),
    .STYLE("AUTO")
)
rd_hash (
    .data_in(i_dst_mac),
    .state_in(32'hffffffff),
    .data_out(),
    .state_out(dst_mac_hash)
);

lfsr #(
    .LFSR_WIDTH(32),
    .LFSR_POLY(32'h4c11db7),
    .LFSR_CONFIG("GALOIS"),
    .LFSR_FEED_FORWARD(0),
    .REVERSE(1),
    .DATA_WIDTH(48),
    .STYLE("AUTO")
)
wr_hash (
    .data_in(i_src_mac),
    .state_in(32'hffffffff),
    .data_out(),
    .state_out(src_mac_hash)
);

assign wr_addr_next = src_mac_hash[TABLE_ADDR_WIDTH-1:0];
assign rd_addr_next = dst_mac_hash[TABLE_ADDR_WIDTH-1:0];

logic [47:0] mac_key;

always_ff @(posedge core_clk) begin
    if (core_rst) begin
        wr_addr <= '0;
        rd_addr <= '0;
        access_d <= '0;
    end else begin
        wr_addr <= wr_addr_next;
        rd_addr <= rd_addr_next;
        access_d <= {access_d[0], i_access};

        if (access_d[0]) begin
            mac_table_mem[wr_addr] <= {i_src_mac, i_src_port};
            {mac_key, o_dst_port} <= mac_table_mem[rd_addr];
        end
    end
end

always_comb begin
    o_dst_port_valid = (mac_key == i_dst_mac) & access_d[1];
end

endmodule