module output_demux #(
    parameter ETHERNET_COUNT = 4
) (
    input                   core_clk,
    input                   core_rst,

    input                   eth_clk,
    input                   eth_rst,

    axis_intf.SLAVE         s_core_axis,
    axis_intf.MASTER        m_eth_axis[ETHERNET_COUNT]
);

localparam CORE_WIDTH = s_core_axis.DATA_WIDTH;

axis_intf #(.DATA_WIDTH(CORE_WIDTH)) demux_axis[ETHERNET_COUNT]();
axis_intf #(.DATA_WIDTH(CORE_WIDTH)) broadcast_axis[ETHERNET_COUNT]();
cpuif_intf #(.ADDR_WIDTH(6)) fifo_reg_cpuif[ETHERNET_COUNT]();


// we should convert back to little endian here.

axis_output_demux #(
    .M_COUNT(ETHERNET_COUNT)
) u_axis_output_demux (
    .i_clk      (core_clk),
    .i_rst      (core_rst),

    .s_axis     (s_core_axis),
    .m_axis     (demux_axis),
    .m_axis_broadcast(broadcast_axis)
);


// could make these not SAFs and just convert fifos.
generate
    for (genvar i = 0; i < ETHERNET_COUNT; i++) begin : gen_upscale_fifo
        axis_intf #(.DATA_WIDTH(CORE_WIDTH)) muxed_axis();

        axis_arb_mux #(
            .S_COUNT(2),
            .DATA_WIDTH(64),
            .ARB_TYPE_ROUND_ROBIN(0),
            .ID_ENABLE(0),
            .S_ID_WIDTH(0),
            .UPDATE_TID(0)
        ) u_axis_arb_mux (
            .clk            (core_clk),
            .rst            (core_rst),

            .s_axis_tdata   ({broadcast_axis[i].tdata, demux_axis[i].tdata}),
            .s_axis_tkeep   ({broadcast_axis[i].tkeep, demux_axis[i].tkeep}),
            .s_axis_tvalid  ({broadcast_axis[i].tvalid, demux_axis[i].tvalid}),
            .s_axis_tready  ({broadcast_axis[i].tready, demux_axis[i].tready}),
            .s_axis_tlast   ({broadcast_axis[i].tlast, demux_axis[i].tlast}),
            .s_axis_tid     ('0),
            .s_axis_tdest   ('0),
            .s_axis_tuser   ({broadcast_axis[i].tuser, demux_axis[i].tuser}),

            .m_axis_tdata   (muxed_axis.tdata),
            .m_axis_tkeep   (muxed_axis.tkeep),
            .m_axis_tvalid  (muxed_axis.tvalid),
            .m_axis_tready  (muxed_axis.tready),
            .m_axis_tlast   (muxed_axis.tlast),
            .m_axis_tid     (),
            .m_axis_tdest   (),
            .m_axis_tuser   (muxed_axis.tuser)
        );

        upscale_saf_wrapper #(
            .DEPTH(4096)
        ) u_upscale_saf_wrapper (
            .s_clk          (core_clk),
            .s_rst          (core_rst),

            .m_clk          (eth_clk),
            .m_rst          (eth_rst),

            .s_axis         (muxed_axis),
            .m_axis         (m_eth_axis[i]),

            .rclr           ('0),
            .reg_cpuif      (fifo_reg_cpuif[i])
        );
    end
endgenerate

endmodule