module switch_core #(
    parameter ETHERNET_COUNT = 4
) (
    input                   core_clk,
    input                   core_rst,

    input                   eth_clk,
    input                   eth_rst,


    axis_intf.SLAVE         s_eth_axis[ETHERNET_COUNT],
    axis_intf.MASTER        m_eth_axis[ETHERNET_COUNT]
);

axis_intf #(.DATA_WIDTH(64), .ID_WIDTH(8)) input_axis();

axis_intf #(.DATA_WIDTH(64), .ID_WIDTH(8)) mac_lookup_axis();

axis_intf #(.DATA_WIDTH(64), .ID_WIDTH(8)) output_axis();

input_mux #(
    .ETHERNET_COUNT(ETHERNET_COUNT)
) u_input_mux (
    .core_clk       (core_clk),
    .core_rst       (core_rst),

    .eth_clk        (eth_clk),
    .eth_rst        (eth_rst),

    .s_eth_axis     (s_eth_axis),
    .m_core_axis    (input_axis)
);

mac_lookup u_mac_lookup (
    .core_clk       (core_clk),
    .core_rst       (core_rst),

    .s_axis         (input_axis),
    .m_axis         (mac_lookup_axis)
);

assign output_axis.tdata        = mac_lookup_axis.tdata;
assign output_axis.tkeep        = mac_lookup_axis.tkeep;
assign output_axis.tvalid       = mac_lookup_axis.tvalid;
assign mac_lookup_axis.tready = output_axis.tready;
assign output_axis.tlast        = mac_lookup_axis.tlast;
assign output_axis.tdest        = mac_lookup_axis.tid;
assign output_axis.tid          = mac_lookup_axis.tid;
assign output_axis.tuser        = mac_lookup_axis.tuser;

output_demux #(
    .ETHERNET_COUNT (ETHERNET_COUNT)
) u_output_demux (
    .core_clk       (core_clk),
    .core_rst       (core_rst),

    .eth_clk        (eth_clk),
    .eth_rst        (eth_rst),

    .s_core_axis    (mac_lookup_axis),
    .m_eth_axis     (m_eth_axis)
);  

endmodule