module upscale_saf_wrapper #(
    parameter DEPTH = 4096
) (
    input                   s_clk,
    input                   s_rst,

    input                   m_clk,
    input                   m_rst,

    axis_intf.SLAVE         s_axis,
    axis_intf.MASTER        m_axis,

    input                   rclr,
    cpuif_intf.SLAVE        reg_cpuif
);


upscale_saf_wrapper_regs_pkg::upscale_saf_wrapper_regs__in_t hwif_in;
upscale_saf_wrapper_regs_pkg::upscale_saf_wrapper_regs__out_t hwif_out;

assign hwif_in.s_status_overflow.count.hwclr    = rclr & hwif_out.s_status_overflow.count.swacc;
assign hwif_in.s_status_bad_frame.count.hwclr   = rclr & hwif_out.s_status_bad_frame.count.swacc;
assign hwif_in.s_status_good_frame.count.hwclr  = rclr & hwif_out.s_status_good_frame.count.swacc;
assign hwif_in.m_status_overflow.count.hwclr    = rclr & hwif_out.m_status_overflow.count.swacc;
assign hwif_in.m_status_bad_frame.count.hwclr   = rclr & hwif_out.m_status_bad_frame.count.swacc;
assign hwif_in.m_status_good_frame.count.hwclr  = rclr & hwif_out.m_status_good_frame.count.swacc;



// This will result in some bad CDC violations for the frame pulses.
// It is also causing issues with top level sims.
// upscale_saf_wrapper_regs u_upscale_saf_wrapper_regs(
//     .clk                    (m_clk),
//     .rst                    (m_rst),

//     .s_cpuif_req            (reg_cpuif.req),
//     .s_cpuif_req_is_wr      (reg_cpuif.req_is_wr),
//     .s_cpuif_addr           (reg_cpuif.addr),
//     .s_cpuif_wr_data        (reg_cpuif.wr_data),
//     .s_cpuif_wr_biten       (reg_cpuif.wr_biten),
//     .s_cpuif_req_stall_wr   (),
//     .s_cpuif_req_stall_rd   (),
//     .s_cpuif_rd_ack         (reg_cpuif.rd_ack),
//     .s_cpuif_rd_err         (),
//     .s_cpuif_rd_data        (reg_cpuif.rd_data),
//     .s_cpuif_wr_ack         (reg_cpuif.wr_ack),
//     .s_cpuif_wr_err         (),

//     .hwif_in                (hwif_in),
//     .hwif_out               (hwif_out)
// );

logic [31:0]    s_status_depth;
logic [31:0]    s_status_depth_commit;
logic           s_status_overflow;
logic           s_status_bad_frame;
logic           s_status_good_frame;
logic [31:0]    m_status_depth;
logic [31:0]    m_status_depth_commit;
logic           m_status_overflow;
logic           m_status_bad_frame;
logic           m_status_good_frame;

assign hwif_in.s_status_depth.val.next         = s_status_depth; 
assign hwif_in.s_status_depth_commit.val.next  = s_status_depth_commit; 
assign hwif_in.s_status_overflow.count.incr    = s_status_overflow; 
assign hwif_in.s_status_bad_frame.count.incr   = s_status_bad_frame; 
assign hwif_in.s_status_good_frame.count.incr  = s_status_good_frame; 
assign hwif_in.m_status_depth.val.next         = m_status_depth; 
assign hwif_in.m_status_depth_commit.val.next  = m_status_depth_commit; 
assign hwif_in.m_status_overflow.count.incr    = m_status_overflow; 
assign hwif_in.m_status_bad_frame.count.incr   = m_status_bad_frame; 
assign hwif_in.m_status_good_frame.count.incr  = m_status_good_frame; 

axis_async_fifo_adapter #(
    .DEPTH(DEPTH),
    .S_DATA_WIDTH(s_axis.DATA_WIDTH),
    .M_DATA_WIDTH(m_axis.DATA_WIDTH),
    .OUTPUT_FIFO_ENABLE(1),
    .FRAME_FIFO(1),
    .DROP_BAD_FRAME(1),
    .DROP_WHEN_FULL(1)
) u_axis_async_fifo_adapter (
    .s_clk                  (s_clk),
    .s_rst                  (s_rst),

    .s_axis_tdata           (s_axis.tdata),
    .s_axis_tkeep           (s_axis.tkeep),
    .s_axis_tvalid          (s_axis.tvalid),
    .s_axis_tready          (s_axis.tready),
    .s_axis_tlast           (s_axis.tlast),
    .s_axis_tid             (s_axis.tid),
    .s_axis_tdest           (s_axis.tdest),
    .s_axis_tuser           (s_axis.tuser),

    .m_clk                  (m_clk),
    .m_rst                  (m_rst),

    .m_axis_tdata           (m_axis.tdata),
    .m_axis_tkeep           (m_axis.tkeep),
    .m_axis_tvalid          (m_axis.tvalid),
    .m_axis_tready          (m_axis.tready),
    .m_axis_tlast           (m_axis.tlast),
    .m_axis_tid             (m_axis.tid),
    .m_axis_tdest           (m_axis.tdest),
    .m_axis_tuser           (m_axis.tuser),

    // Don't use pause reqs
    .s_pause_req            ('0),
    .s_pause_ack            (),
    .m_pause_req            ('0),
    .m_pause_ack            (),

    // BAD CDC FIX THIS!!!!!
    // verilator lint_off WIDTHTRUNC
    .s_status_depth         (s_status_depth),
    .s_status_depth_commit  (s_status_depth_commit),
    // verilator lint_on WIDTHTRUNC
    .s_status_overflow      (s_status_overflow),
    .s_status_bad_frame     (s_status_bad_frame),
    .s_status_good_frame    (s_status_good_frame),
    // verilator lint_off WIDTHTRUNC
    .m_status_depth         (m_status_depth),
    .m_status_depth_commit  (m_status_depth_commit),
    // verilator lint_on WIDTHTRUNC
    .m_status_overflow      (m_status_overflow),
    .m_status_bad_frame     (m_status_bad_frame),
    .m_status_good_frame    (m_status_good_frame)
);


endmodule