module switch_fpga #(
    parameter N_PORTS = 4,
    parameter TARGET = "XILINX"
)(
    input i_clk_100,
    input i_rst_100,

    input i_clk_125,
    input i_rst_125,

    input  wire [N_PORTS-1:0]       rgmii_rx_clk,
    input  wire [N_PORTS*4-1:0]     rgmii_rxd,
    input  wire [N_PORTS-1:0]       rgmii_rx_ctl,
    output wire [N_PORTS-1:0]       rgmii_tx_clk,
    output wire [N_PORTS*4-1:0]     rgmii_txd,
    output wire [N_PORTS-1:0]       rgmii_tx_ctl
);

axis_intf #(.DATA_WIDTH(16)) eth_rx_axis[N_PORTS]();
axis_intf #(.DATA_WIDTH(16)) eth_tx_axis[N_PORTS]();


axil_intf reg_axil();

assign reg_axil.awvalid = '0;
assign reg_axil.wvalid = '0;
assign reg_axil.rready = '0;
assign reg_axil.bready = '0;

ethernet_top #(
    .N_PORTS(N_PORTS),
    .TARGET(TARGET)
) u_ethernet_top (
    .i_clk_100          (i_clk_100),
    .i_rst_100          (i_rst_100),

    .i_clk_125          (i_clk_125),
    .i_clk_125_90       (),
    .i_rst_125          (i_rst_125),

    .m_eth_axis         (eth_rx_axis),
    .s_eth_axis         (eth_tx_axis),

    .rgmii_rx_clk       (rgmii_rx_clk),
    .rgmii_rxd          (rgmii_rxd),
    .rgmii_rx_ctl       (rgmii_rx_ctl),
    .rgmii_tx_clk       (rgmii_tx_clk),
    .rgmii_txd          (rgmii_txd),
    .rgmii_tx_ctl       (rgmii_tx_ctl),

    .s_eth_reg_axil     (reg_axil)
);

// change core clock to 200 after we fix CDC in the hwif interface
switch_core #(
    .ETHERNET_COUNT(N_PORTS)
) u_switch_core (
    .core_clk           (i_clk_100),
    .core_rst           (i_rst_100),

    .eth_clk            (i_clk_100),
    .eth_rst            (i_rst_100),

    .s_eth_axis         (eth_rx_axis),
    .m_eth_axis         (eth_tx_axis)
);

endmodule