module switch_fpga_top #(
    parameter N_PORTS = 4,
    parameter TARGET = "XILINX"
)(
    input i_clk_50,
    input i_rstn,

    input  wire [N_PORTS-1:0]       rgmii_rx_clk,
    input  wire [N_PORTS*4-1:0]     rgmii_rxd,
    input  wire [N_PORTS-1:0]       rgmii_rx_ctl,
    output wire [N_PORTS-1:0]       rgmii_tx_clk,
    output wire [N_PORTS*4-1:0]     rgmii_txd,
    output wire [N_PORTS-1:0]       rgmii_tx_ctl
);

logic clk_100;
logic clk_125;

clk_wiz_0 u_clk_wiz_0
(
    .clk_in1(i_clk_50),

    .clk_out1(clk_100),
    .clk_out2(clk_125)
);

logic rst_100;
logic rst_125;

logic rst_100_sr[2:0];
always @(posedge clk_100) begin
    rst_100_sr <= {rst_100_sr[1:0], ~i_rstn};
end
assign rst_100 = rst_100_sr[2];

logic rst_125_sr[2:0];
always @(posedge clk_125) begin
    rst_125_sr <= {rst_125_sr[1:0], ~i_rstn};
end
assign rst_125 = rst_125_sr[2];

switch_fpga #(
    .N_PORTS(N_PORTS),
    .TARGET(TARGET)
) u_switch_fpga (
    .i_clk_100          (clk_100),
    .i_rst_100          (rst_100),

    .i_clk_125          (clk_125),
    .i_rst_125          (rst_125),

    .rgmii_rx_clk       (rgmii_rx_clk),
    .rgmii_rxd          (rgmii_rxd),
    .rgmii_rx_ctl       (rgmii_rx_ctl),
    .rgmii_tx_clk       (rgmii_tx_clk),
    .rgmii_txd          (rgmii_txd),
    .rgmii_tx_ctl       (rgmii_tx_ctl)
);

endmodule